package locasto.monocrack;

import java.io.*;

import java.util.*;

public class Driver{

    private InputStreamReader isr;
    private BufferedReader keyboard;

	private String dataFileName;

	//------------ Interceptors

	private static Normalize normalize = new Normalize();
	private static AposRule aposRule = new AposRule();
	private static CharCounter charCounter = new CharCounter();
	private static CipherChart cipherChart = new CipherChart();
	private static PlainChart plainChart = new PlainChart();
	private static AlphabetLoader alphabetLoader = new AlphabetLoader();
	
	private static CommonWords commonWords = new CommonWords();

	public Driver( String [] args ) throws Exception{

		dataFileName = args[0]; //   ../../data/alphabet.x.properties


		System.err.println( "" );
		isr = new InputStreamReader(System.in);
		keyboard = new BufferedReader(isr);
		String input=null;
		boolean stop=false;

		do{
			System.err.print( "monocrack> " );
			input = keyboard.readLine();
			if( input == null ){
				stop = true;
			}else if( input.equals( "exit" ) ){
				stop = true;
			}else{
				try{
					doInterceptors( input );
				}catch( Exception e ){
					System.err.println( "Input failed: "+e.getMessage() );
				}
			}

		}while( !stop );
		keyboard.close();
		System.out.println( "" );
	}

	private void doInterceptors( String input ) throws Exception {

		normalize.process( input );
		String normalizedText = normalize.getNormalizedText();

		aposRule.process( normalizedText );
		String strippedNormalizedText = aposRule.getStrippedOutput();
		Vector singleApos = aposRule.getSingleLetter();
		Vector doubleApos = aposRule.getDoubleLetter();
		System.out.println( "\nApostrophe Rule output:" );
		for( int i=0;i<singleApos.size();i++ ){
			System.out.println( ((String)singleApos.elementAt( i )) );
		}
		for( int i=0;i<doubleApos.size();i++ ){
			System.out.println( ((String)doubleApos.elementAt( i )) );
		}
		System.out.println( "\nNormalized Ciphertext:" );
		System.out.println( strippedNormalizedText );
		
		System.out.println( "\n  CipherText frequency chart:\n" );
		charCounter.process( strippedNormalizedText );
		int [] cipherTextFrequency = charCounter.getFrequency();
		cipherChart.process( cipherTextFrequency, System.out );

		System.out.println( "\n" );

		System.out.println( "\n  PlainText frequency chart:\n" );
		alphabetLoader.process( dataFileName );
		int [] plainTextFrequency = alphabetLoader.getFrequency();
		plainChart.process( plainTextFrequency, System.out );

		//print out a translation table based solely on frequency
		//eg, K->{D,L}
		System.out.println( "\nStraight Frequency Match:" );

		//find 'single' words
		System.out.println( "\nSingle Words (A or I):" );

		//find 'double letter' words
		System.out.println( "\nDouble Letter Words:" );

		//find common three letter words
		System.out.println( "\nThree Letter Words:" );

		//match frequencies between common english words and the previous
		//collections

		//if the apos rule has helped us, then use it

		//print out ciphertext with plaintext right underneath

	}

	public static void main( String [] args ){
		try{
			new Driver( args );
		}catch( Exception e ){
			System.err.println( "Problem: "+e.getMessage() );
		}
	}

}
